@@ -1,12 +1,16 @@
-from ckan.lib.search import SearchIndexError
-from splitgill.indexing.utils import DOC_TYPE
-from splitgill.search import create_version_query
+import copy
+import json
 from elasticsearch import NotFoundError
 from elasticsearch_dsl import MultiSearch, Search
+from splitgill.indexing.utils import DOC_TYPE
+from splitgill.search import create_version_query
 
+from ckan.lib.search import SearchIndexError
+from ckan.plugins import PluginImplementations
 from .. import common
 from ..datastore_utils import prefix_resource, prefix_field
 from ..importing.details import get_all_details
+from ...interfaces import IVersionedDatastore
 
 
 def run_search(search, indexes, version=None):
@@ -184,3 +188,56 @@ def get_fields(resource_id, version=None):
     field_cache[cache_key] = (mapping, fields)
 
     return mapping, fields
+
+
+def convert_to_multisearch(query):
+    # save a copy of the original query
+    basic_query = copy.deepcopy(query)
+    multisearch_query = {}
+
+    # allow other plugins to modify the query before processing, e.g. to remove any
+    # custom filters
+    for plugin in PluginImplementations(IVersionedDatastore):
+        query = plugin.datastore_before_convert_basic_query(query)
+
+    if 'q' in query:
+        multisearch_query['search'] = query['q']
+
+    if 'filters' in query:
+        filter_list = []
+        for field, values in query['filters'].items():
+            if not isinstance(values, list):
+                values = [values]
+            if field == '__geo__':
+                for value in values:
+                    if isinstance(value, str):
+                        value = json.loads(value)
+                    if value['type'] == 'Polygon':
+                        filter_list.append({'geo_custom_area': [value['coordinates']]})
+                    else:
+                        # I cannot find any examples of anything other than polygons, so
+                        # I'm not sure it was ever implemented for the old searches
+                        raise NotImplemented
+            else:
+                subgroup = []
+                subgroup_count = 0
+                for value in values:
+                    if field != '' and value != '':
+                        subgroup.append(
+                            {'string_equals': {'fields': [field], 'value': value}}
+                        )
+                        subgroup_count += 1
+                if subgroup_count > 1:
+                    filter_list.append({'or': subgroup})
+                elif subgroup_count == 1:
+                    filter_list += subgroup
+        multisearch_query['filters'] = {'and': filter_list}
+
+    # allow plugins to modify the output, with the additional context of the original
+    # basic query
+    for plugin in PluginImplementations(IVersionedDatastore):
+        multisearch_query = plugin.datastore_after_convert_basic_query(
+            basic_query, multisearch_query
+        )
+
+    return multisearch_query
